import QtQuick 2.15
import QtMultimedia 5.15
import QtQuick.Controls 2.15
import QtQuick.Controls.Universal 2.15

import SignsSynthesis 1.0

Rectangle {
    id: root

    property bool neverScrolledUp: true
    property var translations: ({})

    readonly property double contentTextHeight: height / 25
    readonly property string mainColor: "#F6B233"
    readonly property int dissappearTimeout: hideTextTime
    readonly property string fontColor: "#black"
    readonly property string grayColor: "#B2B2B2"
    readonly property string darkGrayColor: "#555555"

    readonly property double leftOffsett: contentTextHeight

    function showText( text, isPhrase, isKeyboardInput, translateToGestures ) {
        const now = new Date( );
        listModel.append( { sentence: text,
                            time: now,
                            isPhrase: isPhrase,
                            isKeyboardInput: isKeyboardInput } );
        clearScreenTimer.restart( );

        console.log( "content height: " + listView.contentHeight );
        console.log( "content y: " + listView.contentY );
        console.log( "height: " + listView.height );
        console.log( "delta: " + ( listView.contentY + listView.height - listView.contentHeight ) );
        console.log( "scroll step: " + scrollingTimer.scrollStep );

        if( root.neverScrolledUp ) {
            console.log( "Scrolling to the last added element." );
            listView.positionViewAtEnd( );
        }

        if( translateToGestures ) {
            recognitionModule.synthesizeSignsLanguage( text, isKeyboardInput, remoteOutput.width / remoteOutput.height )
        }
    }

    Component.onCompleted: {
        recognitionModule.phraseRecognized.connect( function( phrase ) {
            root.showText( phrase, true, false, true );
        } );
        recognitionModule.translated.connect( function( text, translation ) {
            console.log("New translation for text '" + text + "': " + translation)
            if(text !== translation) {
                root.translations[text] = translation;
                root.translationsChanged();
                if( root.neverScrolledUp ) {
                    console.log( "Scrolling to the last added element." );
                    listView.positionViewAtEnd( );
                }
            }
        } );
        recognitionModule.systemMessage.connect( function( text ) {
            root.showText( text, false, false, false );
        } );
        recognitionModule.textUp.connect( function( start ) {
            root.neverScrolledUp = false;
            scrollingTimer.running = start;
        } );
        recognitionModule.textDown.connect( function( ) {
            root.neverScrolledUp = true;
            scrollingTimer.running = false;
            listView.positionViewAtEnd( );
        } );
        recognitionModule.clearText.connect( function( ) {
            listModel.clear( );
            root.translations = {}
        } );
        recognitionModule.removeLastPhrase.connect( function( ) {
            if( listModel.count > 0 ) {
                listModel.remove(listModel.count - 1);
                recognitionModule.removeLastPhraseFromUsbLog( );
            }
        } );
        recognitionModule.showUserTexts.connect( function( texts ) {
            for( let i = 0; i < texts.length; ++i ) {
                root.showText( texts[ i ], false, true, true );
            }
        } );
        recognitionModule.currentBrailleInputTextChanged.connect( function( newText ) {
            edit.text = newText;
            edit.cursorPosition = newText.length;
        } );
        recognitionModule.partialRecognitionResultChanged.connect( () => {
            if(recognitionModule.partialRecognitionResult.length > 0 && !scrollingTimer.running) {
                listView.positionViewAtEnd( );
            }
        } );

        recognitionModule.connectNewBluetoothDevice( );
    }

    Timer {
        id: scrollingTimer
        interval: 100
        repeat: true

        readonly property double scrollStep: listView.height / 20

        onTriggered: {
            if( listView.contentY <= 0 ) {
                return;
            }
            const newContentY = Math.max( listView.contentY - scrollStep, 0 );
            listView.contentY = newContentY;
        }
    }

    Timer {
        id: clearScreenTimer
        interval: 10 * 60 * 1000

        onTriggered: {
            listModel.clear( );
            recognitionModule.clearVisibleTexts( );
        }
    }

    Timer {
        running: true
        interval: 60 * 1000
        repeat: true

        onTriggered: {
            const now = new Date( );
            let i = 0;
            for( ; i < listModel.count; ++i ) {
                const item = listModel.get( i );
                const diff = now - item.time;
                if( diff < root.dissappearTimeout ) {
                    break;
                }
            }
            if( i > 0 ) {
                listModel.remove( 0, i );
            }
        }
    }

    Timer {
        running: true
        interval: 3000
        repeat: true

        property int previousConnectionStatus: 0

        onTriggered: {
            let connectionStatus;
            if( recognitionModule.isEthernetConnected( ) ) {
                connectionImage.source = "images/ethernet_on.svg";
                connectionStatus = 2;
            } else {
                if( recognitionModule.isWifFiConnected( ) ) {
                    connectionImage.source = "images/wifi_on.svg";
                    connectionStatus = 1;
                } else {
                    connectionImage.source = "images/no_connection.svg";
                    connectionStatus = 0;
                }
            }
            if( connectionStatus !== previousConnectionStatus ) {
                previousConnectionStatus = connectionStatus;
                recognitionModule.sendHardwareStatusToCloud( );
            }
        }
    }

    Rectangle {
        id: header
        width: parent.width
        height: parent.height * 0.125
        color: root.mainColor
        z: 1000

        anchors.top: parent.top

        Image {
            id: titleIcon
            source: "images/title_icon.svg"
            height: parent.height * 0.6
            width: height

            sourceSize.width: width
            sourceSize.height: height

            anchors.verticalCenter: parent.verticalCenter
            anchors.left: parent.left
            anchors.leftMargin: root.leftOffsett
        }

        Text {
            text: recognitionModule.userName
            color: root.fontColor

            font.pixelSize: root.contentTextHeight

            anchors.verticalCenter: parent.verticalCenter
            anchors.left: titleIcon.right
            anchors.leftMargin: root.leftOffsett
        }

        Rectangle {
            color: root.mainColor
            height: parent.height

            anchors.verticalCenter: parent.verticalCenter
            anchors.right: parent.right
            anchors.left: buttonsRow.left
        }

        Row {
            id: buttonsRow
            spacing: imageHeight / 3

            anchors.verticalCenter: parent.verticalCenter
            anchors.right: parent.right
            anchors.rightMargin: root.leftOffsett

            readonly property double imageHeight: header.height * 0.5

            Image {
                source: recognitionModule.pushToRecognize ?
                        ( recognitionModule.recordingAllowed ?
                          "images/push_to_talk_listening.svg" : "images/push_to_talk_not_listening.svg" ) :
                        ( recognitionModule.recordingAllowed ? "images/listening.svg" : "images/not_listening.svg")
                height: parent.imageHeight
                width: height

                sourceSize.width: width
                sourceSize.height: height

                anchors.verticalCenter: parent.verticalCenter
            }

            Image {
                source: recognitionModule.recordingAllowed ?
                        ( recognitionModule.isUsingMicrophoneArray ?
                          "images/microphone_array.svg" : "images/single_microphone.svg" ) :
                        ( recognitionModule.isUsingMicrophoneArray ?
                          "images/microphone_array_disabled.svg" : "images/single_microphone_disabled.svg" )
                height: parent.imageHeight
                width: height

                sourceSize.width: width
                sourceSize.height: height

                anchors.verticalCenter: parent.verticalCenter
            }

            Image {
                id: connectionImage
                source: "images/no_connection.svg"
                height: parent.imageHeight
                width: height

                sourceSize.width: width
                sourceSize.height: height

                anchors.verticalCenter: parent.verticalCenter
            }

            Image {
                source: recognitionModule.bluetoothConnected ?
                        "images/bluetooth_connected.svg" : "images/bluetooth_not_connected.svg"
                height: parent.imageHeight
                width: height

                sourceSize.width: width
                sourceSize.height: height

                anchors.verticalCenter: parent.verticalCenter
            }

            Image {
                source: recognitionModule.brailleDisplayConnected ?
                        "images/braille_connected.svg" : "images/braille_not_connected.svg"
                height: parent.imageHeight
                width: height

                sourceSize.width: width
                sourceSize.height: height

                anchors.verticalCenter: parent.verticalCenter
            }

            Image {
                source: qrCodePath
                height: header.height * 0.6
                width: height
                visible: recognitionModule.onlineChatEnabled

                anchors.verticalCenter: parent.verticalCenter
            }
        }
    }

    Rectangle {
        id: gesturesView
        color: "black"
        width: root.width * 0.27
        visible: recognitionModule.enableSignsTranslation

        anchors.top: header.bottom
        anchors.bottom: mainWindow.top
        anchors.left: mainWindow.left
        anchors.topMargin: root.leftOffsett
        anchors.bottomMargin: root.leftOffsett

        VideoOutput {
            id: remoteOutput
            visible: recognitionModule.signsSynthesisState === SignsSynthesisFacade.CallState.STREAMING
            fillMode: VideoOutput.PreserveAspectCrop
            source: recognitionModule.remoteVideoRenderer

            anchors.fill: parent
        }

        BusyIndicator {
            running: true
            visible: recognitionModule.signsSynthesisState === SignsSynthesisFacade.CallState.WAITING_FOR_STREAMER ||
                     recognitionModule.signsSynthesisState === SignsSynthesisFacade.CallState.ESTABLISHING_VIDEO_CONNECTION
            Universal.accent: root.mainColor
            width: parent.width / 5
            height: width

            anchors.centerIn: parent
        }
    }

    ListView {
        id: listView
        interactive: false

        anchors.top: header.bottom
        anchors.bottom: mainWindow.top
        anchors.left: recognitionModule.enableSignsTranslation ? gesturesView.right : parent.left
        anchors.leftMargin: root.leftOffsett
        anchors.rightMargin: root.leftOffsett
        anchors.right: parent.right
        anchors.topMargin: root.contentTextHeight
        anchors.bottomMargin: mainWindow.height

        readonly property double rowSpacing: root.width / 40
        readonly property double textMarkerWidth: rowSpacing / 4
        readonly property double fontSize: root.contentTextHeight * recognitionModule.fontScale

        delegate: Row {
            width: listView.width
            spacing: listView.rowSpacing

            Rectangle {
                id: textMarker
                color: isKeyboardInput ? root.mainColor :
                                         ( isPhrase ? root.grayColor : root.darkGrayColor )
                width: listView.textMarkerWidth
                height: textItem.height
                radius: width / 3

                Rectangle {
                    color: parent.color
                    width: parent.width / 2

                    anchors.top: parent.top
                    anchors.bottom: parent.bottom
                    anchors.right: parent.right
                }
            }

            Column {
                width: listView.width - listView.rowSpacing - listView.textMarkerWidth
                spacing: root.contentTextHeight / 2

                Text {
                    id: textItem
                    font.pixelSize: listView.fontSize
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: isKeyboardInput || isPhrase ? Text.AlignLeft : Text.AlignHCenter
                    text: sentence
                    wrapMode: Text.WordWrap
                    color: root.fontColor

                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                }

                Text {
                    font.pixelSize: listView.fontSize
                    verticalAlignment: Text.AlignVCenter
                    text: root.translations[sentence] ?? ""
                    wrapMode: Text.WordWrap
                    color: root.fontColor
                    font.italic: true
                    visible: root.translations.hasOwnProperty(sentence)

                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                }

                Rectangle {
                    height: root.contentTextHeight / 2
                    color: 'white'

                    anchors {
                        left: parent.left
                        right: parent.right
                    }
                }
            }
        }

        footer: Row {
            id: footer

            anchors.left: parent.left
            anchors.right: parent.right

            Rectangle {
                color: root.grayColor
                width: listView.textMarkerWidth
                height: vadImage.height
                radius: width / 3

                Rectangle {
                    color: parent.color
                    width: parent.width / 2

                    anchors.top: parent.top
                    anchors.bottom: parent.bottom
                    anchors.right: parent.right
                }
            }

            Item {
                width: listView.rowSpacing
                height: vadImage.height
            }

            Image {
                id: vadImage
                visible: recognitionModule.activeSoundSourceFound
                source: "images/vad_on.svg"
                height: root.height / 12
                width: height

                sourceSize.width: width
                sourceSize.height: height
            }

            Text {
                font.pixelSize: listView.fontSize
                verticalAlignment: Text.AlignVCenter
                text: recognitionModule.partialRecognitionResult
                wrapMode: Text.WordWrap
                color: root.fontColor
                width: footer.width - vadImage.width - listView.textMarkerWidth - listView.rowSpacing
            }
        }

        model: ListModel {
            id: listModel
        }

        ScrollBar.vertical: ScrollBar {
            id: textScrollBar
            contentItem.opacity: listView.contentHeight > listView.height ? 1 : 0
        }
    }

    Rectangle {
        id: mainWindow
        color: root.mainColor
        width: parent.width - 2 * root.leftOffsett
        height: textInputArea.height + 2 * borderWidth

        anchors.bottom: parent.bottom
        anchors.bottomMargin: root.leftOffsett
        anchors.horizontalCenter: parent.horizontalCenter

        readonly property real borderWidth: 5

        Rectangle {
            id: textInputArea
            color: "white"
            height: header.height * 0.8

            anchors.left: parent.left
            anchors.leftMargin: parent.borderWidth
            anchors.right: parent.right
            anchors.rightMargin: parent.borderWidth
            anchors.bottom: parent.bottom
            anchors.bottomMargin: parent.borderWidth

            TextInput {
                id: edit
                focus: true
                clip: true
                color: root.fontColor
                font.pixelSize: 0.9 * textInputArea.height - 20

                anchors.fill: parent
                anchors.margins: 10

                function handleEnterKey( ) {
                    const text = edit.text;
                    if( text.length > 0 ) {
                        root.showText( text, false, true, true );
                        recognitionModule.addKeyboardInput( text );
                        edit.clear( );
                    }
                }

                Keys.onReturnPressed: handleEnterKey( )
                Keys.onEnterPressed: handleEnterKey( )

                property string placeholderText: qsTr( "Ввода текста / Printed text" )

                Text {
                    text: parent.placeholderText
                    color: "#9d9d9d"
                    visible: !parent.text
                    font.pixelSize: root.contentTextHeight

                    anchors.verticalCenter: parent.verticalCenter
                    anchors.left: parent.left
                }

                Keys.onPressed: function(event) {
                    if( event.key === Qt.Key_HomePage && !event.isAutoRepeat ) {
                        if(recognitionModule.pushToRecognize) {
                            recognitionModule.recordingAllowed = true;
                        } else {
                            recognitionModule.recordingAllowed = !recognitionModule.recordingAllowed;
                        }
                        return;
                    }
                    event.accepted = false;
                }
                Keys.onReleased: function(event) {
                    if( ( event.key === Qt.Key_HomePage ||
                          event.key === Qt.Key_Escape ) && !event.isAutoRepeat ) {
                        if(recognitionModule.pushToRecognize) {
                            recognitionModule.recordingAllowed = false;
                        }
                        return;
                    }
                    event.accepted = false;
                }
                Keys.onEscapePressed: function(event) {
                    if(!event.isAutoRepeat) {
                        if(recognitionModule.pushToRecognize) {
                            recognitionModule.recordingAllowed = true;
                        } else {
                            recognitionModule.recordingAllowed = !recognitionModule.recordingAllowed;
                        }
                    }
                }
            }
        }
    }
}
